#-------------------------------------------------------------------------------------------------
# Copyright (c) International Business Machines Corp., 2009
#
# IBM XIV Storage System Software
#
# Licensed materials provided under the terms of the IBM International Program
# License Agreement. See the Software
# licensing materials that came with the IBM XIV Storage System for terms and
# conditions.
#
#------------------------------------------------------------------------------------------------- 

import sys, time, logging
from xiv_xsf.console import print_in_color, get_console_dimensions
from xml.etree.ElementTree import SubElement, Element, tostring
import output, table

class ColoredTable(object):
    COLORS = ['cyan','dark_cyan']
    
    def __init__(self, fields, devices, xiv_specific=True):
        self.fields = fields
        self.devices = devices
        self.xiv_specific = xiv_specific
    
    def get_length_of_logest_field(self):
        max = 1
        for field in self.fields:
            if max <= field.length:
                max = field.length
        return max

    def get_console_dimensions(self):
        return get_console_dimensions()
        
    def get_number_of_dynamic_columns(self):
        number_of_dynamic_columns = 0
        for field in self.fields:
            if (not self.xiv_specific) and field.xiv_specific:
                continue
            if not field.length:
                number_of_dynamic_columns+=1
        return number_of_dynamic_columns

    def get_number_of_used_fields(self):
        counter=0
        for field in self.fields:
            if (not self.xiv_specific) and field.xiv_specific:
                continue
            counter+=1
        return counter
 
    def get_number_and_size_of_fixed_columns(self):
        number_of_fixed_columns = 0
        size_of_fixed_columns = 0
        for field in self.fields:
            if (not self.xiv_specific) and field.xiv_specific:
                continue
            if field.length:
                number_of_fixed_columns+=1
                size_of_fixed_columns+=field.length
        return (number_of_fixed_columns, size_of_fixed_columns)

    def get_maximum_length_for_dynamic_column(self):
        (cols, lines) = get_console_dimensions()
        total_number_of_columns = self.get_number_of_used_fields()
        number_of_dynamic_columns = self.get_number_of_dynamic_columns()
        (number_of_fixed_columns, size_of_fixed_columns) = self.get_number_and_size_of_fixed_columns()
        
        max_column_length = cols
        max_column_length -= size_of_fixed_columns + 2*(number_of_fixed_columns)
        max_column_length -= 2*(number_of_dynamic_columns)
        if (number_of_dynamic_columns + number_of_fixed_columns)>0:
            max_column_length += 2
        if number_of_dynamic_columns > 0:
            max_column_length = max_column_length / number_of_dynamic_columns
        return max_column_length

    def get_length_of_data_of_longest_line(self):
        if len(self.rows) == 0:
           return 0
        max_column_length = [0 for item in self.rows[0]]
        for row in self.rows:
            for index in range(0,len(row)):
                length = len(row[index])
                if length > max_column_length[index]:
                    max_column_length[index] = length
        max_row_length = 0
        for item in max_column_length:
            max_row_length+=item
        return max_row_length

    def get_length_of_longest_line(self):
        max_row_length = self.get_length_of_data_of_longest_line()
        total_number_of_columns = self.get_number_of_used_fields()

        if total_number_of_columns>0:
            return max_row_length+2*total_number_of_columns-2
        else:
            return max_row_length

    def all_rows_fit(self):
        (cols, lines) = get_console_dimensions()
        total_number_of_columns = self.get_number_of_used_fields()
        max_row_length = self.get_length_of_longest_line()
        
        diff = cols
        diff -= max_row_length
        if diff>=0:
            return True
        else:
            return False
    
    def get_flat_column_length(self):
        (cols, lines) = get_console_dimensions()
        total_number_of_columns = self.get_number_of_used_fields()
        if total_number_of_columns>0:
            return (cols-2*(total_number_of_columns)+2)/total_number_of_columns
        else:
            return cols

    def get_maximum_length_for_column(self):
        (cols, lines) = get_console_dimensions()
        
        dynamic_length = self.get_maximum_length_for_dynamic_column()
        flat_length = self.get_flat_column_length()

        if self.all_rows_fit():
            return cols
    
        if flat_length > dynamic_length:
            return flat_length
        else:
            return dynamic_length

    def get_fields_names(self):
        names = []
        for field in self.fields:
            if (not self.xiv_specific) and field.xiv_specific:
                continue
            names.append(field.displayname)
        return names

    def get_raw_table_header_and_rows(self):
        header = self.get_fields_names()
        rows = []
        for device in self.devices:
            row = []
            if (device == {}):
                continue # Device has no data at all
            for field in self.fields:
                value = ''
                if (not self.xiv_specific) and field.xiv_specific:
                    continue
                try:
                    if device[field.name] is not None:
                        value = str(device[field.name])
                    else:
                        value = "N/A"
                except KeyError, e:
                    value = "N/A"
                row.append(value)
            rows.append(row)
        return (header, rows)

    def get_table(self):
        (header, rows) = self.get_raw_table_header_and_rows()
        if len(rows)>0 and len(rows[0])==0:
            return None
        self.rows = [header,] + rows
        max_len = self.get_maximum_length_for_column()
        output_table = table.indent([header] + rows, True, separateRows=True, delim = '  ',
                                    wrapfunc=lambda x: table.wrap_onspace_strict(x, max_len))
        return output_table
   
    def is_break_line(self, line):
        return '-' in line and ' ' not in line
    
    def output(self):
        self.output_table = self.get_table()
        if self.output_table is None:
            return None

        self.output_table = self.output_table.splitlines()
        first_row = self.output_header()
        self.output_rows(first_row)
    
    def output_header(self):
        print_in_color(self.output_table[0], 'white')
        for index in range(1,len(self.output_table)):
            line = self.output_table[index]
            print_in_color(line, 'white')
            if self.is_break_line(line):
                return index+1

    def output_rows(self, first_row):
        colors = enumerate(self.COLORS)
        print_color = colors.next()[1]
        for index in range(first_row, len(self.output_table)):
            line = self.output_table[index]
            if '-' in line and ' ' not in line:
                print_in_color(line, 'white')
                while True:
                    try:
                        print_color = colors.next()[1]
                        break
                    except StopIteration:
                        colors = enumerate(self.COLORS)
                continue
            if line.strip() == '':
                continue
            if line.replace('|','').strip() == '':
                continue
            print_in_color(line, print_color)
        
class CommaSeperatedValues(object):
    def __init__(self, fields, devices, delimiter=','):
        self.fields = fields
        self.devices = devices
        self.delimiter = delimiter
    
    def output(self):
        for field in self.fields:
            sys.stdout.write("%s%s" % (field.displayname.strip(), self.delimiter)),
        print
        for device in self.devices:
            if (device == {}):
                continue # Device has no data at all
            for field in self.fields:
                try:
                    if device[field.name] is not None:
                        sys.stdout.write("%s%s" % (device[field.name], self.delimiter)),
                    else:
                        sys.stdout.write(self.delimiter),
                except KeyError, e:
                    sys.stdout.write(self.delimiter),
            print

class XmlFormat(object):
    def __init__(self, fields, devices):
        self.fields = fields
        self.devices = devices

    def prepare_xml(self):
        xml_root = Element("DevList")
        for device in self.devices:
            if (device == {}):
                continue # Device has no data at all
            device_element = SubElement(xml_root, "DeviceItem")
            for field in self.fields:
                value = ''
                try:
                    if device[field.name] is not None:
                        value = str(device[field.name])
                    else:
                        value = "N/A"
                except KeyError, e:
                    value = "N/A"
                attribute_element = SubElement(device_element, field.name.strip())
                attribute_element.text = value
        return xml_root
    
    def output(self):
        xml_root = self.prepare_xml()
        print tostring(xml_root)
